@@ -1,16 +1,18 @@
-use crate::parser::Tokenizer;
+use std::convert::TryFrom;
+
+use crate::models::Ledger;
 use crate::Error;
 use crate::{
     models::{Currency, HasName},
     CommonOpts,
 };
 use std::ops::Deref;
-use std::path::PathBuf;
 
-pub fn execute(path: PathBuf, options: &CommonOpts) -> Result<(), Error> {
-    let mut tokenizer: Tokenizer = Tokenizer::from(&path);
-    let items = tokenizer.tokenize(options);
-    let ledger = items.to_ledger(options)?;
+pub fn execute(options: &CommonOpts, maybe_ledger: Option<Ledger>) -> Result<(), Error> {
+    let ledger = match maybe_ledger {
+        Some(ledger) => ledger,
+        None => Ledger::try_from(options)?,
+    };
     let mut commodities = ledger
         .commodities
         .iter()
