@@ -398,3 +398,32 @@ fn reg_exchange() {
 
     test_args(args);
 }
+
+#[test]
+/// Check the exchange option in the register report
+fn roi() {
+    let args = &[
+        "roi",
+        "--init-file",
+        "tests/example_files/empty_ledgerrc",
+        "-f",
+        "tests/example_files/hledger_roi.ledger",
+        "--cash-flows",
+        "cash",
+        "--assets-value",
+        "snake",
+        "-Q",
+    ];
+    let assert_1 = Command::cargo_bin("dinero").unwrap().args(args).assert();
+    let output = String::from_utf8(assert_1.get_output().to_owned().stdout).unwrap();
+
+    for (i, line) in output.lines().into_iter().enumerate() {
+        match i {
+            3 => assert!(String::from(line).contains("2.50%")),
+            5 => assert!(String::from(line).contains("2.38%")),
+            _ => assert!(true),
+        }
+    }
+
+    test_args(args);
+}
